import { useState, useEffect } from "react";
import { useNavigate } from "react-router-dom";
import Autocomplete from "@mui/material/Autocomplete";
import TextField from "@mui/material/TextField";
import Box from "@mui/material/Box";
import Button from "@mui/material/Button";
import Chip from "@mui/material/Chip";
import Select from "@mui/material/Select";
import MenuItem from "@mui/material/MenuItem";
import InputLabel from "@mui/material/InputLabel";
import FormControl from "@mui/material/FormControl";
import { useCreateServer, useVersionList } from "@mcl/queries";

const defaultServer = {
  version: "latest",
  serverType: "VANILLA",
  memory: "512",
};

export default function Create() {
  const [spec, setSpec] = useState(defaultServer);
  const nav = useNavigate();
  const versionList = useVersionList();
  const [versions, setVersions] = useState(["latest"]);
  const createServer = useCreateServer(spec);

  const updateSpec = (attr, val) => {
    const s = { ...spec };
    s[attr] = val;
    setSpec(s);
    console.log(s);
  };

  useEffect(() => {
    if (!versionList.data) return;
    setVersions([
      "latest",
      ...versionList.data.versions
        .filter(({ type: releaseType }) => releaseType === "release")
        .map(({ id }) => id),
    ]);
  }, [versionList.data]);

  const coreUpdate = (attr) => (e) => updateSpec(attr, e.target.value);

  async function upsertSpec() {
    if (validateSpec() !== "validated") return;
    createServer(spec)
      .then(() => nav("/"))
      .catch(alert);
  }

  function validateSpec() {
    console.log("TODO CREATE VALIDATION");
    if (!spec.host) return alertValidationError("Host cannot be blank");
    if (!spec.name) return alertValidationError("Name not included");
    if (!spec.version) return alertValidationError("Version cannot be blank");
    return "validated";
  }

  function alertValidationError(reason) {
    alert(`Could not validate spec because: ${reason}`);
  }

  return (
    <Box
      className="create-options"
      sx={{ width: "100%", maxWidth: "600px", margin: "auto" }}
    >
      <FormControl fullWidth sx={{ mt: "2rem", display: "flex", gap: ".5rem" }}>
        <TextField
          label="Name"
          onChange={coreUpdate("name")}
          helperText="Example: My Survival World"
          defaultValue={spec.name}
          FormHelperTextProps={{ sx: { ml: 0 } }}
          required
        />
        <TextField
          label="Host"
          onChange={coreUpdate("host")}
          helperText="Example: host.mydomain.com"
          FormHelperTextProps={{ sx: { ml: 0 } }}
          required
        />
        <TextField
          label="Version"
          onChange={coreUpdate("version")}
          value={spec.version}
          select
          required
          SelectProps={{ MenuProps: { sx: { maxHeight: "20rem" } } }}
        >
          {versions.map((v, k) => (
            <MenuItem value={v} key={k}>
              {v}
            </MenuItem>
          ))}
        </TextField>
        <TextField
          label="Server Type"
          onChange={coreUpdate("serverType")}
          value={spec.serverType}
          select
          required
        >
          <MenuItem value={"VANILLA"}>Vanilla</MenuItem>
          <MenuItem value={"FABRIC"}>Fabric</MenuItem>
          <MenuItem value={"PAPER"}>Paper</MenuItem>
          <MenuItem value={"SPIGOT"}>Spigot</MenuItem>
        </TextField>

        <TextField
          label="Memory"
          onChange={coreUpdate("memory")}
          defaultValue={spec.memory}
          required
        />
        <Button onClick={upsertSpec} variant="contained">
          Create
        </Button>
      </FormControl>
    </Box>
  );
}
